*************************************************************************************************
**            Written by Dino Papararo            08-December-2001
**
**  FUNCTION
**
**    Julian68K -- perform Z = (Z^(2^n)) + C  (1 <= n <= 11)
**
**  SYNOPSIS
**
**    WORD Julian68K (ULONG Iterations,ULONG Power,double Cre,double Cim,double JKre,double JKim)
**
**
**  DESCRIPTION
**
**  This function tests if a point belongs or not at Julia set
**
**  Optimized for pipelines of 68882+ coprocessors
**
**  NOTICE: ALL VARIABLES ARE INTO REGISTERS FOR FULL SPEEEED
**
**  d0:Iterations d1:Power
**
**  fp0:Cre/Zr fp1:Cim/Zi fp2:JKre fp3:JKim fp4:Zr2 fp5:Zi2/Dist fp6:MaxDist fp7:NotUsed ;-)
*************************************************************************************************

*        MACHINE  68060


        XDEF  _Julian68K

        section data

Radius  dc.x  4.0            * Store Radius in data mem
Power   dc.w  0              * Store Power in data mem

        section code

_Julian68K:

        fmove.x Radius,fp6   * Save Radius from mem to fp6
        move.w  d1,Power     * Save Power from d1 to mem
        bra.w   PowerLoop    * Go to PowerLoop

Start:
        move.w  Power,d1     * Load Power from mem to d1

PowerLoop:

        fmove.x fp1,fp5      * zi2   =  zi
        fmul.x  fp0,fp1      * zi    =  zr * zi
        fmul.x  fp0,fp0      * zr    =  zr * zr
        fmul.x  fp5,fp5      * zi2  *=  zi2
        fmove.x fp0,fp4      * zr2   = zr
        fadd.x  fp1,fp1      * zi   += zi
        fsub.x  fp5,fp0      * zr   -= zi2
        dbra.w  d1,PowerLoop * if --Power != -1 go to PowerLoop

        fadd.x  fp4,fp5      * Dist = zr2 + zi2
        fcmp.x  fp6,fp5      * Compare  MaxDist & Dist
        fbgt.w  End          * if Dist > MaxDist then go to End

        fadd.x  fp3,fp1      * zi   += JKim
        fadd.x  fp2,fp0      * zr   += JKre
        dbra.w  d0,Start     * if --Iterations != -1 go to Start
        clr.w   d0           * Iterations = 0

End:

        rts                  * return Iterations

        end

