************************************************************************************
**            Written by Dino Papararo            12-December-2001
**
**  FUNCTION
**
**    MandelnPPC -- perform Z = (Z^(2^n)) + C  (1 <= n <= 11)
**
**  SYNOPSIS
**
**    ULONG MandelnPPC (ULONG Iterations,ULONG Power,long double Cre,long double Cim)
**
**
**  This function tests if a point belongs or not at mandelbrot's set
**
**  Optimized for pipelines of PPC processors.
**
**  r3:Iterations r4:Power
**  f1:Cre f2:Cim f3:Zr f4:Zi f5:Zr2 f6:Zi2 f7:Dist f8:MaxDist
************************************************************************************


                include powerpc/ppcmacros.i
                include powerpc/powerpc.i

;                xref _PowerPCBase

                xdef _MandelnPPC

                section code

                cpu POWERPC

_MandelnPPC

                 prolog             ; start

                 lf     f8,Radius   ; MaxDist = Radius
                 fmr    f4,f2       ; Zi = Cim
                 fmr    f3,f1       ; Zr = Cre
                 b+     .Loop2      ; go to Loop2

.Loop1
                 mfctr  r3          ; Iterations = Counter
.Loop2
                 mtctr  r4          ; Counter = Power
.Loop3

                 fmul   f6,f4,f4    ; zi2 = zi * zi
                 fmul   f5,f3,f3    ; zr2 = zr * zr
                 fmul   f4,f3,f4    ; zi = zr * zi
                 fsub   f3,f5,f6    ; zr = zr2 - zi2
                 fadd   f4,f4,f4    ; zi = zi + zi
                 bdnz   .Loop3      ; if --Counter > 0 go to Loop3

                 mtctr  r3          ; Counter = Iterations
                 fadd   f7,f5,f6    ; Dist = zr2 + zi2
                 fadd   f3,f3,f1    ; zr = zr + Cre
                 fadd   f4,f4,f2    ; zi = zi + Cim
                 fcmpu  f7,f8       ; compare Dist with MaxDist
                 bdnzf+ GT,.Loop1   ; if Dist < MaxDist and --Counter > 0 go to Loop1
****************************************************************************************

.End

                 mfctr  r3          ; Iterations = Counter

                 epilog             ; end


                save
                data

             align.d
Radius          dc.d   4.0

                restore
